import { use, assert, expect } from 'chai';
import chaiAlmost from 'chai-almost';
import { sumBase, sumStat, rebin, histogramify } from "../src/helpers/uhepp.js";

use(chaiAlmost())

describe("sumBase", () => {
    const toy_yield = {
      "process_a": {
        "base": [ 0,  1,  2,  3,  4,  5],
        "stat": [0.1, 0.1, 0.1, 0.1, 0.1]
      },
      "process_b": {
        "base": [10, 12, 14, 16, 18, 20],
        "stat": [1.0, 1.2, 1.4, 1.6, 1.8, 2.0]
      },
      "process_c": {
        "base": [ 4,  4,  -5,  -5,  -6,  5],
        "stat": [ 0.4, 0.4, 0.5, 0.5, 0.6, 0.6]
      }
    }

    it("should return 0 if the yield object is empty", () => {
        const sum = sumBase({}, ["process_a", "process_b"])
        assert.equal(sum, 0);
    })
    it("should return 0 if the yield non of the processes are present", () => {
        const sum = sumBase(toy_yield, ["process_d", "process_e"])
        assert.equal(sum, 0);
    })
    it("should return the total sum of the single process found in the object", () => {
        const sum = sumBase(toy_yield, ["process_a", "process_e"])
        assert.equal(sum, 15);
    })
    it("should return the total sum of all process", () => {
        const sum = sumBase(toy_yield, ["process_a", "process_c"])
        assert.equal(sum, 12);
    })
    it("should return 0 if process name list is empty", () => {
        const sum = sumBase(toy_yield, [])
        assert.equal(sum, 0);
    })
    it("should return 0 if process name list and object are empty", () => {
        const sum = sumBase({}, [])
        assert.equal(sum, 0);
    })

    it("should return 0 if the yield object is empty for bin=2", () => {
        const sum = sumBase({}, ["process_a", "process_b"], 2)
        assert.equal(sum, 0);
    })
    it("should return 0 if the yield non of the processes are present for and bin=2", () => {
        const sum = sumBase(toy_yield, ["process_d", "process_e"], 2)
        assert.equal(sum, 0);
    })
    it("should return the sum of the single process found in the object for bin=2", () => {
        const sum = sumBase(toy_yield, ["process_a", "process_e"], 2)
        assert.equal(sum, 2);
    })
    it("should return the sum of all process for bin=2", () => {
        const sum = sumBase(toy_yield, ["process_a", "process_c"], 2)
        assert.equal(sum, -3);
    })
    it("should return 0 if process name list is empty for bin=2", () => {
        const sum = sumBase(toy_yield, [], 2)
        assert.equal(sum, 0);
    })
    it("should return 0 if process name list and object are empty for bin=2", () => {
        const sum = sumBase({}, [], 2)
        assert.equal(sum, 0);
    })
});

describe("sumStat", () => {
    const toy_yield = {
      "process_a": {
        "base": [ 0,  1,  2,  3,  4,  5],
        "stat": [0.1, 0.1, 0.1, 0.1, 0.1]
      },
      "process_b": {
        "base": [10, 12, 14, 16, 18, 20],
        "stat": [1.0, 1.2, 1.4, 1.6, 1.8, 2.0]
      },
      "process_c": {
        "base": [ 4,  4,  -5,  -5,  -6,  5],
        "stat": [ 0.4, 0.4, 0.5, 0.5, 0.6, 0.6]
      }
    }

    it("should return 0 if the yield object is empty", () => {
        const sum = sumStat({}, ["process_a", "process_b"])
        assert.equal(sum, 0);
    })
    it("should return 0 if the yield non of the processes are present", () => {
        const sum = sumStat(toy_yield, ["process_d", "process_e"])
        assert.equal(sum, 0);
    })
    it("should return the total sum of the single process found in the object", () => {
        const sum = sumStat(toy_yield, ["process_a", "process_e"])
        expect(sum * sum).to.almost.equal(0.05);
    })
    it("should return the total sum of all process", () => {
        const sum = sumStat(toy_yield, ["process_a", "process_c"])
        expect(sum * sum).to.almost.equal(1.59);
    })
    it("should return 0 if process name list is empty", () => {
        const sum = sumStat(toy_yield, [])
        assert.equal(sum, 0);
    })
    it("should return 0 if process name list and object are empty", () => {
        const sum = sumStat({}, [])
        assert.equal(sum, 0);
    })

    it("should return 0 if the yield object is empty for bin=2", () => {
        const sum = sumStat({}, ["process_a", "process_b"], 2)
        assert.equal(sum, 0);
    })
    it("should return 0 if the yield non of the processes are present for and bin=2", () => {
        const sum = sumStat(toy_yield, ["process_d", "process_e"], 2)
        assert.equal(sum, 0);
    })
    it("should return the sum of the single process found in the object for bin=2", () => {
        const sum = sumStat(toy_yield, ["process_a", "process_e"], 2)
        expect(sum).to.almost.equal(0.1);
    })
    it("should return the sum of all process for bin=2", () => {
        const sum = sumStat(toy_yield, ["process_a", "process_c"], 2)
        assert.equal(sum * sum, 0.26);
    })
    it("should return 0 if process name list is empty for bin=2", () => {
        const sum = sumStat(toy_yield, [], 2)
        assert.equal(sum, 0);
    })
    it("should return 0 if process name list and object are empty for bin=2", () => {
        const sum = sumStat({}, [], 2)
        assert.equal(sum, 0);
    })
});

describe("rebin", () => {
    const toy_series = [-3,  1,  2,  4,  8, 16, 32, 64]
    const old_edges  =    [0, 10, 20, 30, 40, 50, 60] 

    it("should return the original values if both edges are identical", () => {
      const rebinned = rebin(toy_series, old_edges, old_edges)
      assert.deepEqual(rebinned, toy_series)
    })
    it("should raise an exception if the new edges are not a subset", () => {
      assert.throws(
        () => rebin(toy_series, old_edges, [0, 5, 15, 60]),
        /subset/
      )
    })
    it("should sum the entries of merged bins", () => {
      const new_edges = [0, 20, 50, 60]
      const rebinned = rebin(toy_series, old_edges, new_edges)
      assert.deepEqual(rebinned, [-3, 3, 28, 32, 64])
    })
    it("should sum all entries for a single bin output", () => {
      const new_edges = [0, 60]
      const rebinned = rebin(toy_series, old_edges, new_edges)
      assert.deepEqual(rebinned, [-3, 63, 64])
    })
    it("should discard bins outside new edges", () => {
      const new_edges = [10, 30, 50]
      const rebinned = rebin(toy_series, old_edges, new_edges)
      assert.deepEqual(rebinned, [-2, 6, 24, 96])
    })
    it("should raise an exception if the input lengths are incompatible", () => {
      const new_edges = [10, 30, 50]
      assert.throws(
        () => rebin(toy_series, new_edges, new_edges),
        /number/
      )
    })
    it("should apply pre and post to merged bin values", () => {
      const new_edges = [0, 20, 50, 60]
      const rebinned = rebin(
        toy_series,
        old_edges,
        new_edges,
        (x) => x + 1,
        (x) => x * x,
      )
      assert.deepEqual(rebinned, [4, 25, 31*31, 33*33, 65*65])
    })
})

describe("histogramify", () => {
    const edges  =     [0, 10, 20, 30, 40]
    const toy_series =   [1,  2,  4,  8]

    it("should return correct new y values", () => {
      const [x, y] = histogramify(edges, toy_series)
      assert.deepEqual(x, [0, 10, 10, 20, 20, 30, 30, 40])
    })
    it("should return correct new x values", () => {
      const [x, y] = histogramify(edges, toy_series)
      assert.deepEqual(y, [1, 1, 2, 2, 4, 4, 8, 8])
    })
    it("should throw an exception if length are incompatible", () => {
      assert.throws(
        () => histogramify([0, 10, 20], [1, 2, 3]),
        /number/
      )
    })

})
